@@ -3,16 +3,14 @@
 
 #pragma once
 
-#include "../desktopio/terminal.hpp"
 #include "../desktopio/application.hpp"
+#include "../desktopio/terminal.hpp"
 
 namespace netxs::events::userland
 {
     struct term
     {
-        using mime = ansi::clip::mime;
-
-        EVENTPACK( term, netxs::events::userland::root::custom )
+        EVENTPACK( term, ui::e2::extra::slot3 )
         {
             EVENT_XS( cmd    , si32 ),
             GROUP_XS( preview, si32 ),
@@ -24,6 +22,7 @@ namespace netxs::events::userland
             {
                 EVENT_XS( align    , si32 ),
                 EVENT_XS( wrapln   , si32 ),
+                EVENT_XS( io_log   , bool ),
                 GROUP_XS( selection, si32 ),
                 GROUP_XS( colors   , rgba ),
 
@@ -42,6 +41,7 @@ namespace netxs::events::userland
             {
                 EVENT_XS( align    , si32 ),
                 EVENT_XS( wrapln   , si32 ),
+                EVENT_XS( io_log   , bool ),
                 GROUP_XS( selection, si32 ),
                 GROUP_XS( colors   , rgba ),
 
@@ -80,7 +80,6 @@ namespace netxs::app::term
     static constexpr auto desc = "Desktopio Terminal";
 
     using events = netxs::events::userland::term;
-    using mime = clip::mime;
 
     namespace
     {
@@ -96,6 +95,12 @@ namespace netxs::app::term
                 item.reflow();
             }
         }
+        static auto _update_gear(ui::pads& boss, menu::item& item, hids& gear)
+        {
+            auto& look = item.views[item.taken];
+            gear.set_tooltip(look.notes, true);
+            _update(boss, item);
+        }
         static auto _update_to(ui::pads& boss, menu::item& item, si32 i)
         {
             item.select(i);
@@ -112,7 +117,6 @@ namespace netxs::app::term
                     if (item.views.size())
                     {
                         item.taken = (item.taken + 1) % item.views.size();
-                        _update(boss, item);
                     }
                     if (gear.capture(boss.id))
                     {
@@ -129,6 +133,10 @@ namespace netxs::app::term
                         });
                         gear.dismiss(true);
                     }
+                    if (item.views.size())
+                    {
+                        _update_gear(boss, item, gear);
+                    }
                 };
                 boss.LISTEN(tier::release, hids::events::mouse::button::up::left, gear)
                 {
@@ -138,7 +146,7 @@ namespace netxs::app::term
                     if (item.views.size() && item.taken)
                     {
                         item.taken = 0;
-                        _update(boss, item);
+                        _update_gear(boss, item, gear);
                     }
                 };
                 boss.LISTEN(tier::release, e2::form::state::mouse, active)
@@ -161,7 +169,7 @@ namespace netxs::app::term
                     proc(boss, item, gear);
                     if constexpr (AutoUpdate)
                     {
-                        if (item.brand == menu::item::Option) _update(boss, item);
+                        if (item.brand == menu::item::Option) _update_gear(boss, item, gear);
                     }
                     gear.dismiss(true);
                 };
@@ -186,10 +194,10 @@ namespace netxs::app::term
             { menu::type::Option,   menu::item::Option   },
             { menu::type::Repeat,   menu::item::Repeat   }};
 
-        #define PROC_LIST \
+        #define proc_list \
             X(Noop                      ) /* */ \
             X(TerminalQuit              ) /* */ \
-            X(TerminalMaximize          ) /* */ \
+            X(TerminalFullscreen        ) /* */ \
             X(TerminalRestart           ) /* */ \
             X(TerminalSendKey           ) /* */ \
             X(TerminalWrapMode          ) /* */ \
@@ -199,7 +207,8 @@ namespace netxs::app::term
             X(TerminalFindPrev          ) /* */ \
             X(TerminalUndo              ) /* Undo/Redo for cooked read under win32 */ \
             X(TerminalRedo              ) /* */ \
-            X(TerminalPaste             ) /* */ \
+            X(TerminalClipboardPaste    ) /* */ \
+            X(TerminalClipboardWipe     ) /* */ \
             X(TerminalSelectionCopy     ) /* */ \
             X(TerminalSelectionMode     ) /* */ \
             X(TerminalSelectionRect     ) /* Linear/Rectangular */ \
@@ -215,6 +224,7 @@ namespace netxs::app::term
             X(TerminalViewportTop       ) /* */ \
             X(TerminalViewportEnd       ) /* */ \
             X(TerminalViewportCopy      ) /* */ \
+            X(TerminalStdioLog          ) /* */ \
             X(TerminalLogStart          ) /* */ \
             X(TerminalLogPause          ) /* */ \
             X(TerminalLogStop           ) /* */ \
@@ -236,14 +246,14 @@ namespace netxs::app::term
         enum func
         {
             #define X(_proc) _proc,
-            PROC_LIST
+            proc_list
             #undef X
         };
 
         static const auto route_options = std::unordered_map<text, func>
         {
             #define X(_proc) { #_proc, func::_proc },
-            PROC_LIST
+            proc_list
             #undef X
         };
 
@@ -319,14 +329,14 @@ namespace netxs::app::term
             {
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
-                    boss.RISEUP(tier::release, e2::form::quit, boss.This());
+                    boss.SIGNAL(tier::anycast, app::term::events::cmd, ui::term::commands::ui::commands::sighup);
                 });
             }
-            static void TerminalMaximize(ui::pads& boss, menu::item& item)
+            static void TerminalFullscreen(ui::pads& boss, menu::item& item)
             {
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
-                    boss.RISEUP(tier::release, e2::form::maximize, gear);
+                    boss.RISEUP(tier::release, e2::form::layout::fullscreen, gear);
                 });
             }
             static void TerminalRestart(ui::pads& boss, menu::item& item)
@@ -350,13 +360,20 @@ namespace netxs::app::term
                     boss.SIGNAL(tier::anycast, app::term::events::cmd, ui::term::commands::ui::commands::redo);
                 });
             }
-            static void TerminalPaste(ui::pads& boss, menu::item& item)
+            static void TerminalClipboardPaste(ui::pads& boss, menu::item& item)
             {
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
                     boss.SIGNAL(tier::anycast, app::term::events::data::paste, gear);
                 });
             }
+            static void TerminalClipboardWipe(ui::pads& boss, menu::item& item)
+            {
+                _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
+                {
+                    gear.clear_clipboard();
+                });
+            }
             static void TerminalSelectionCopy(ui::pads& boss, menu::item& item)
             {
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
@@ -406,18 +423,14 @@ namespace netxs::app::term
             {
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
-                    auto info = e2::form::upon::scroll::bypage::y.param();
-                    info.vector = 1;
-                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bypage::y, info);
+                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bypage::y, info, ({ .vector = 1 }));
                 });
             }
             static void TerminalViewportPageDown(ui::pads& boss, menu::item& item)
             {
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
-                    auto info = e2::form::upon::scroll::bypage::y.param();
-                    info.vector = -1;
-                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bypage::y, info);
+                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bypage::y, info, ({ .vector = -1 }));
                 });
             }
             static void TerminalViewportLineUp(ui::pads& boss, menu::item& item)
@@ -425,9 +438,7 @@ namespace netxs::app::term
                 item.reindex([](auto& utf8){ auto v = xml::take<si32>(utf8); return v ? v.value() : 1; });
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
-                    auto info = e2::form::upon::scroll::bystep::y.param();
-                    info.vector = std::abs(item.views[item.taken].value);
-                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bystep::y, info);
+                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bystep::y, info, ({ .vector = std::abs(item.views[item.taken].value) }));
                 });
             }
             static void TerminalViewportLineDown(ui::pads& boss, menu::item& item)
@@ -435,43 +446,35 @@ namespace netxs::app::term
                 item.reindex([](auto& utf8){ auto v = xml::take<si32>(utf8); return v ? v.value() : 1; });
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
-                    auto info = e2::form::upon::scroll::bystep::y.param();
-                    info.vector = -std::abs(item.views[item.taken].value);
-                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bystep::y, info);
+                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bystep::y, info, ({ .vector = -std::abs(item.views[item.taken].value) }));
                 });
             }
             static void TerminalViewportTop(ui::pads& boss, menu::item& item)
             {
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
-                    auto info = e2::form::upon::scroll::to_top::y.param();
-                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::to_top::y, info);
+                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::to_top::y, info, ());
                 });
             }
             static void TerminalViewportEnd(ui::pads& boss, menu::item& item)
             {
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
-                    auto info = e2::form::upon::scroll::to_end::y.param();
-                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::to_end::y, info);
+                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::to_end::y, info, ());
                 });
             }
             static void TerminalViewportPageLeft(ui::pads& boss, menu::item& item)
             {
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
-                    auto info = e2::form::upon::scroll::bypage::x.param();
-                    info.vector = 1;
-                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bypage::x, info);
+                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bypage::x, info, ({ .vector = 1 }));
                 });
             }
             static void TerminalViewportPageRight(ui::pads& boss, menu::item& item)
             {
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
-                    auto info = e2::form::upon::scroll::bypage::x.param();
-                    info.vector = -1;
-                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bypage::x, info);
+                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bypage::x, info, ({ .vector = -1 }));
                 });
             }
             static void TerminalViewportCharLeft(ui::pads& boss, menu::item& item)
@@ -479,9 +482,7 @@ namespace netxs::app::term
                 item.reindex([](auto& utf8){ auto v = xml::take<si32>(utf8); return v ? v.value() : 1; });
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
-                    auto info = e2::form::upon::scroll::bystep::x.param();
-                    info.vector = std::abs(item.views[item.taken].value);
-                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bystep::x, info);
+                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bystep::x, info, ({ .vector = std::abs(item.views[item.taken].value) }));
                 });
             }
             static void TerminalViewportCharRight(ui::pads& boss, menu::item& item)
@@ -489,11 +490,21 @@ namespace netxs::app::term
                 item.reindex([](auto& utf8){ auto v = xml::take<si32>(utf8); return v ? v.value() : 1; });
                 _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
                 {
-                    auto info = e2::form::upon::scroll::bystep::x.param();
-                    info.vector = -std::abs(item.views[item.taken].value);
-                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bystep::x, info);
+                    boss.SIGNAL(tier::anycast, e2::form::upon::scroll::bystep::x, info, ({ .vector = -std::abs(item.views[item.taken].value) }));
                 });
             }
+            static void TerminalStdioLog(ui::pads& boss, menu::item& item)
+            {
+                item.reindex([](auto& utf8){ return xml::take<bool>(utf8).value(); });
+                _submit<true>(boss, item, [](auto& boss, auto& item, auto& gear)
+                {
+                    boss.SIGNAL(tier::anycast, preview::io_log, item.views[item.taken].value);
+                });
+                boss.LISTEN(tier::anycast, release::io_log, state)
+                {
+                    _update_to(boss, item, state);
+                };
+            }
             static void TerminalLogStart(ui::pads& boss, menu::item& item)
             {
 
@@ -567,16 +578,16 @@ namespace netxs::app::term
         static const auto proc_map = std::unordered_map<func, submit_proc>
         {
             #define X(_proc) { func::_proc, &disp::_proc },
-            PROC_LIST
+            proc_list
             #undef X
         };
-        #undef PROC_LIST
+        #undef proc_list
 
         auto list = menu::list{};
         auto defs = menu::item::look{};
         for (auto data_ptr : menudata)
         {
-            auto item_ptr = std::make_shared<menu::item>();
+            auto item_ptr = ptr::shared<menu::item>();
             auto& data = *data_ptr;
             auto& item = *item_ptr;
             auto route = data.take(menu::attr::route, func::Noop,          route_options);
@@ -595,11 +606,7 @@ namespace netxs::app::term
                     .onkey = label->take(menu::attr::onkey, defs.onkey),
                 });
             }
-            if (item.views.empty())
-            {
-                log(ansi::err("term: menu item without label"));
-                continue;
-            }
+            if (item.views.empty()) continue; // Menu item without label.
             auto setup = [route](ui::pads& boss, menu::item& item)
             {
                 if (item.brand == menu::item::Option)
@@ -630,12 +637,13 @@ namespace netxs::app::term
             arg = arg_shadow;
             if (term_type == shared::app_type::normal)
             {
+                //todo revise focus
                 window->plugin<pro::focus>()
                       ->plugin<pro::track>()
                       ->plugin<pro::acryl>()
                       ->plugin<pro::cache>();
             }
-            else window->plugin<pro::focus>(faux);
+            else window->plugin<pro::focus>(pro::focus::mode::focusable, faux);
 
             auto object = window->attach(ui::fork::ctor(axis::Y))
                                 ->colors(cB.fgc(), cB.bgc());
@@ -655,16 +663,15 @@ namespace netxs::app::term
                         // Axis x/y (see XTWINOPS):
                         //   -1 -- preserve
                         //    0 -- maximize (toggle)
-                        if (new_size == dot_00) // Toggle maximize/restore terminal window (only if it is focused by someone).
+                        if (new_size == dot_00) // Toggle fullscreen terminal (only if it is focused by someone).
                         {
-                            auto gates = e2::form::state::keybd::enlist.param();
-                            boss.SIGNAL(tier::anycast, e2::form::state::keybd::enlist, gates);
+                            boss.RISEUP(tier::request, e2::form::state::keybd::enlist, gates, ());
                             if (gates.size())
                             if (auto gate_ptr = bell::getref(gates.back()))
                             {
                                 gate_ptr->SIGNAL(tier::release, e2::form::proceed::onbehalf, [&](auto& gear)
                                 {
-                                    boss.RISEUP(tier::release, e2::form::maximize, gear);
+                                    boss.RISEUP(tier::release, e2::form::layout::fullscreen, gear);
                                 });
                             }
                         }
@@ -678,10 +685,40 @@ namespace netxs::app::term
                 });
 
             auto shell = os::env::shell() + " -i";
-            auto inst = scroll->attach(ui::term::ctor(cwd, arg.empty() ? shell : arg, config));
+            auto inst = scroll->attach(ui::term::ctor(cwd, arg.empty() ? shell : arg, config))
+                              ->plugin<pro::focus>(pro::focus::mode::focused);
             auto scroll_bars = layers->attach(ui::fork::ctor());
             auto vt = scroll_bars->attach(slot::_2, ui::grip<axis::Y>::ctor(scroll));
-            auto hz = term_stat_area->attach(slot::_2, ui::grip_fx2<axis::X>::ctor(scroll))
+            static constexpr auto drawfx = [](auto& boss, auto& canvas, auto handle, auto object_len, auto handle_len, auto region_len, auto wide)
+            {
+                // Brightener isn't suitable for white backgrounds.
+                auto brighter = skin::color(tone::selector);
+                brighter.bga(std::min(0xFF, brighter.bga() * 3));
+                auto brighter_bgc = brighter.bgc();
+                auto boss_bgc = boss.base::color().bgc();
+
+                if (handle_len != region_len) // Show only if it is oversized.
+                {
+                    if (wide) // Draw full scrollbar on mouse hover
+                    {
+                        static auto box = ' ';
+                        canvas.fill([&](cell& c) { c.txt(box).link(boss.bell::id).xlight(); });
+                        canvas.fill(handle, [&](cell& c) { c.bgc().mix(brighter_bgc); });
+                    }
+                    else
+                    {
+                        static auto box = "▄"sv; //"▀"sv;
+                        canvas.fill(handle, [&](cell& c) { c.link(boss.bell::id).bgc().mix(brighter_bgc); });
+                        canvas.fill([&](cell& c) { c.inv(true).txt(box).fgc(boss_bgc); });
+                    }
+                }
+                else
+                {
+                    static auto box = "▄"sv;
+                    canvas.fill([&](cell& c) { c.inv(true).txt(box).fgc(boss_bgc); });
+                }
+            };
+            auto hz = term_stat_area->attach(slot::_2, ui::gripfx<axis::X, drawfx>::ctor(scroll))
                 ->plugin<pro::limit>(twod{ -1,1 }, twod{ -1,1 })
                 ->invoke([&](auto& boss)
                 {
@@ -695,10 +732,43 @@ namespace netxs::app::term
             auto menu = object->attach(slot::_1, slot1);
             cover->invoke([&, &slot1 = slot1](auto& boss) //todo clang 15.0.0 still disallows capturing structured bindings (wait for clang 16.0.0)
             {
-                boss.colors(cell{ cB }.inv(true).txt("▀"sv).link(slot1->id));
-                boss.LISTEN(tier::anycast, app::term::events::release::colors::bg, bg)
+                auto bar = cell{ "▀"sv }.link(slot1->id);
+                auto brush = ptr::shared(cell{ cB }.inv(true).txt("▀"sv).link(slot1->id));
+                auto winsz = ptr::shared(dot_00);
+                auto visible = ptr::shared(slot1->back() != boss.This());
+                auto check_state = ptr::function([state = testy<bool>{ true }, winsz, visible](base& boss) mutable
                 {
-                    boss.color(boss.color().fgc(bg));
+                    if (state(*visible || winsz->y != 1))
+                    {
+                        boss.RISEUP(tier::preview, e2::form::prop::ui::acryl, state.last);
+                        boss.RISEUP(tier::preview, e2::form::prop::ui::cache, state.last);
+                    }
+                });
+                boss.LISTEN(tier::release, e2::form::state::visible, menu_visible, -, (visible, check_state))
+                {
+                    *visible = menu_visible;
+                    (*check_state)(boss);
+                };
+                boss.LISTEN(tier::anycast, e2::form::upon::resized, newsize, -, (winsz, check_state))
+                {
+                    *winsz = newsize;
+                    (*check_state)(boss);
+                };
+                boss.LISTEN(tier::anycast, app::term::events::release::colors::bg, bg, -, (brush))
+                {
+                    brush->fgc(bg);
+                };
+                boss.LISTEN(tier::release, e2::render::any, parent_canvas, -, (bar, winsz, brush))
+                {
+                    if (winsz->y != 1)
+                    {
+                        auto& b = *brush;
+                        parent_canvas.fill([&](cell& c) { c.fusefull(b); });
+                    }
+                    else
+                    {
+                        parent_canvas.fill([&](cell& c) { c.fgc(c.bgc()).fga(0xff).bgc(0).txt(bar).link(bar); });
+                    }
                 };
             });
 
@@ -706,19 +776,19 @@ namespace netxs::app::term
                 ->attach_property(ui::term::events::colors::fg,      app::term::events::release::colors::fg)
                 ->attach_property(ui::term::events::selmod,          app::term::events::release::selection::mode)
                 ->attach_property(ui::term::events::selalt,          app::term::events::release::selection::box)
+                ->attach_property(ui::term::events::io_log,          app::term::events::release::io_log)
                 ->attach_property(ui::term::events::layout::wrapln,  app::term::events::release::wrapln)
                 ->attach_property(ui::term::events::layout::align,   app::term::events::release::align)
                 ->attach_property(ui::term::events::search::status,  app::term::events::search::status)
                 ->invoke([](auto& boss)
                 {
-                    boss.LISTEN(tier::anycast, e2::form::quit, boss_ptr)
+                    boss.LISTEN(tier::anycast, e2::form::proceed::quit::any, fast)
                     {
-                        boss.SIGNAL(tier::preview, e2::form::quit, boss_ptr);
+                        boss.SIGNAL(tier::preview, e2::form::proceed::quit::one, fast);
                     };
-                    boss.LISTEN(tier::preview, e2::form::quit, item_ptr)
+                    boss.LISTEN(tier::preview, e2::form::proceed::quit::one, fast)
                     {
-                        boss.stop();
-                        boss.RISEUP(tier::release, e2::form::quit, item_ptr);
+                        boss.sighup(fast);
                     };
                     boss.LISTEN(tier::anycast, app::term::events::cmd, cmd)
                     {
@@ -759,6 +829,10 @@ namespace netxs::app::term
                     {
                         boss.set_wrapln(wrapln);
                     };
+                    boss.LISTEN(tier::anycast, app::term::events::preview::io_log, state)
+                    {
+                        boss.set_log(state);
+                    };
                     boss.LISTEN(tier::anycast, app::term::events::preview::align, align)
                     {
                         boss.set_align(align);
