@@ -270,7 +270,7 @@ namespace netxs::app::test
 
 
                 // Wikipedia run
-                auto wiki = ansi::esc("\n\n")
+                auto wiki = escx("\n\n")
                     .jet(bias::center).wrp(wrap::off).fgc(clr)
                     .add("Sample Article\n\n")
 
@@ -420,16 +420,16 @@ namespace netxs::app::test
         {
             auto topic = get_text();
             auto window = ui::cake::ctor()
-                ->plugin<pro::focus>()
+                ->plugin<pro::focus>(pro::focus::mode::focused)
                 ->plugin<pro::track>()
                 ->plugin<pro::acryl>()
                 ->plugin<pro::cache>()
                 ->invoke([](auto& boss)
                 {
-                    boss.keybd.accept(true);
-                    boss.LISTEN(tier::anycast, e2::form::quit, item)
+                    //boss.keybd.accept(true);
+                    boss.LISTEN(tier::anycast, e2::form::proceed::quit::any, fast)
                     {
-                        boss.RISEUP(tier::release, e2::form::quit, item);
+                        boss.RISEUP(tier::release, e2::form::proceed::quit::one, fast);
                     };
                 });
             auto object0 = window->attach(ui::fork::ctor(axis::Y))
